@@ -361,7 +361,7 @@ static int xdestroy( XEvent *xevent, XtPointer client )
   history_destroy( conn->transport.priv.x.client ) ;
   remove_allmyeventhandler
     ( XtDisplay( conn->protocol_widget ), conn->transport.priv.x.client ) ;
-  XSafeFlush( XtDisplay( conn->protocol_widget ) ) ;
+  /* XSafeFlush( XtDisplay( conn->protocol_widget ) ) ; */
   return True ;
 }
 
@@ -755,7 +755,7 @@ static void Xim_Destroy( Widget gw )
     ( gw, NoEventMask, True, 
       ( XtEventHandler )Xim_AcceptXService, ( XtPointer )NULL ) ;
   /* ʬȤ˴٤ƥåȤ˽׵᤹롣*/
-  XtCallCallbacks( gw, XtNserverCloseNotify, NULL ) ;
+  /* XtCallCallbacks( gw, XtNserverCloseNotify, NULL ) ; */
   return ;
 }
 
@@ -925,6 +925,7 @@ static int Xim_getSupportedLocales( Widget gw, unsigned char *lstring )
       *dptr ++ = *ptr ;
     }
   }
+  *dptr = '\0';
   if( dptr != startpoint ){
     /* ֺǸʸ "," ʬΥƤʤΤǡǴ
        롣*/
