@@ -472,7 +472,7 @@ int IMStartConversion( IMIC *icp )
 
 void IMStopConversion( IMIC *icp )
 {
-  IMConnection *conn = icp->im->connection ;
+  /* IMConnection *conn = icp->im->connection ; */
 
   if ( ! ( icp->state & IC_CONVERTING ) )
     return ;
@@ -487,11 +487,14 @@ void IMStopConversion( IMIC *icp )
    */
   if( icp->conversion != NULL ){
     detachConverter( icp ) ;
+#if 0
     icp->state &= ~IC_CONVERTING ;
     /* Connection ڤ줿ȤȤ򶵤ʤȤʤ*/
     setEventMask( conn, icp->im->id, icp->id, 0L, 0L ) ;
+#endif
     /* IMFlush( conn ) ; */
   }
+  icp->state &= ~IC_CONVERTING ;
   return ;
 }
 
