@@ -29,6 +29,7 @@
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
+#include <sys/socket.h>
 
 #define main_c
 #include "commondef.h"
@@ -68,6 +69,7 @@ enum {
   OPTION_SKKDIC, OPTION_SKKREC, OPTION_HELP, OPTION_VERSION,
   OPTION_CONFIG, OPTION_XIM_OFF, OPTION_XIM_ON, OPTION_XIMP_OFF,
   OPTION_XIMP_ON,OPTION_AUTOSAVE, OPTION_KINPUT_ON, OPTION_KINPUT_OFF,
+  OPTION_IPv4, OPTION_IPv6,
 } ;
 
 
@@ -110,7 +112,7 @@ extern void initVectorHashTable( void ) ;
 extern void initHenkanKakuteiHashTable( void ) ;
 extern void clearHenkanKakuteiHash( void ) ;
 /* skksoc.c */
-extern int  skkinput_StartCommunication( char *server_name, int port_num ) ;
+extern int  skkinput_StartCommunication( char *server_name, char *service_name, int pf ) ;
 extern int  skkinput_CloseCommunication( void ) ;
 /* skkldic.c */
 extern int  set_localjisyo
@@ -196,7 +198,8 @@ static char *skkinput_config_name ;
  */
 /* ¾skkinput ɬפȤskkInputWidget ˤɬפȤʤ*/
 char *skkserv_host ;
-int  skkserv_portnum ;
+char *skkserv_service ;
+int skkserv_pf = PF_UNSPEC ;
 char *skkinput_local_jisyo_name ;
 char *skkinput_backup_jisyo_name ;
 char *skk_local_jisyo_name ;
@@ -246,7 +249,7 @@ int main( int argc, char *argv[] )
   Arg arg[ 20 ] ;
   Cardinal i ;
   XEvent xevent ;
-  XtIntervalId checkautosave_id ;
+  volatile XtIntervalId checkautosave_id = 0;
   int (*defaultIOErrorHandler)( Display *disp ) ;
 
 #if defined(DEBUG)
@@ -375,7 +378,7 @@ int main( int argc, char *argv[] )
   if( number_of_protocols > 0 ){
 #ifndef MAIKAI_SKKSERV_TO_CONNECT_SURU
     /* socket Ƥ*/
-    skkinput_StartCommunication( skkserv_host, skkserv_portnum ) ;
+    skkinput_StartCommunication( skkserv_host, skkserv_service, skkserv_pf ) ;
 #endif
     /* ɽ꼭ξƤ*/
     set_localjisyo
@@ -552,14 +555,17 @@ static void destroy_protocolServers( void )
 #if defined(SUPPORT_KINPUT)
   if( kinput_protocol != NULL )
     XtDestroyWidget( kinput_protocol ) ;
+  kinput_protocol = NULL;
 #endif
 #if defined(SUPPORT_XIMP)
   if( ximp_protocol != NULL )
     XtDestroyWidget( ximp_protocol ) ;
+  ximp_protocol = NULL;
 #endif
 #if defined(SUPPORT_XIM)
   if( xim_protocol != NULL )
     XtDestroyWidget( xim_protocol ) ;
+  xim_protocol = NULL;
 #endif
   return ;
 }
@@ -595,6 +601,8 @@ static int skkinput_usage( void )
     "-v            or -version",	  "shows version",
     "-h <hostname> or -host <hostname>",  "specifies skkserv host",
     "-p <port_num> or -port <port_num>",  "specifies port number",
+    "-4",				  "specifies to use IPv4",
+    "-6",				  "specifies to use IPv6",
     "-uj <jisyo>   or -userjisyo <jisyo>","specifies skkinput-local-jisyo",
     "-bj <jisyo>   or -backjisyo <jisyo>","specifies skkinput-local-jisyo\'s backup",
     "-sj <jisyo>   or -skkjisyo <jisyo>", "specifies skk-local-jisyo",
@@ -642,6 +650,8 @@ static int skkinput_checkOptions( int *argc, char *argv[], int flag )
     { "-host",		OPTION_HOST,		True },
     { "-p",		OPTION_PORT,		True },
     { "-port",		OPTION_PORT,		True },
+    { "-4",		OPTION_IPv4,		False },
+    { "-6",		OPTION_IPv6,		False },
     { "-uj",		OPTION_USERDIC,		True },
     { "-userjisyo",	OPTION_USERDIC,		True },
     { "-bj",		OPTION_BAKDIC,		True },
@@ -696,10 +706,23 @@ static int skkinput_checkOptions( int *argc, char *argv[], int flag )
     case OPTION_HOST :
       skkserv_host = argv[ i ] ;
       break ;
-      /* skkserv ̿Ѥݡֹꤹ롣*/
+      /* skkserv ̿Ѥ륵ӥ̾/ݡֹꤹ롣*/
     case OPTION_PORT :
-      skkserv_portnum = atoi( argv[ i ] ) ;
+      if (skkserv_service) free(skkserv_service) ;
+      skkserv_service = strdup(argv[i]);
       break ;
+    case OPTION_IPv4:
+      skkserv_pf = PF_INET;
+      break;
+    case OPTION_IPv6:
+#if defined(USE_INET6)
+      skkserv_pf = PF_INET6;
+      break;
+#else
+      fprintf( stderr, "IPv6 not supported.\n" ) ;
+      goto checkoption_errorreturn ;
+#endif
+
       /* ɽ꼭̾ꤹ롣*/
     case OPTION_USERDIC :
       skkinput_local_jisyo_name = argv[ i ] ;
